# -*- coding: utf-8 -*-

import unittest

from datetime import date
from candv import (
    Constants, SimpleConstant, VerboseConstant, Values, ValueConstant,
    VerboseValueConstant,
)


class VerboseConstantTestCase(unittest.TestCase):

    def _create_constant(self, *args, **kwargs):

        class FOO(Constants):
            CONSTANT = VerboseConstant(*args, **kwargs)

        return FOO.CONSTANT

    def test_no_args(self):
        constant = self._create_constant()
        self.assertEqual(constant.name, 'CONSTANT')
        self.assertEqual(constant.full_name, 'FOO.CONSTANT')
        self.assertIsNone(constant.verbose_name)
        self.assertIsNone(constant.help_text)

    def test_versose_name(self):
        constant = VerboseConstant(verbose_name="foo")
        self.assertEqual(constant.verbose_name, "foo")

    def test_help_text(self):
        constant = VerboseConstant(help_text="just test constant")
        self.assertEqual(constant.help_text, "just test constant")

    def test_all_args(self):
        constant = self._create_constant("foo", "just test constant")
        self.assertEqual(constant.name, 'CONSTANT')
        self.assertEqual(constant.full_name, 'FOO.CONSTANT')
        self.assertEqual(constant.verbose_name, "foo")
        self.assertEqual(constant.help_text, "just test constant")

    def test_to_primitive(self):
        constant = self._create_constant()
        self.assertEqual(
            constant.to_primitive(),
            {
                'name': 'CONSTANT',
                'verbose_name': None,
                'help_text': None,
            }
        )

        constant = self._create_constant("Constant", "A test constant")
        self.assertEqual(
            constant.to_primitive(),
            {
                'name': 'CONSTANT',
                'verbose_name': "Constant",
                'help_text': "A test constant",
            }
        )


class ConstantsTestCase(unittest.TestCase):

    def test_one_constant_class(self):

        class FOO(Constants):
            two = SimpleConstant()
            one = SimpleConstant()

        self.assertEqual(
            [x.name for x in FOO.iterconstants()],
            ['two', 'one', ]
        )

    def test_one_constant_name(self):

        class FOO(Constants):
            one = SimpleConstant()
            one = SimpleConstant()

        self.assertEqual([x.name for x in FOO.iterconstants()], ['one', ])

    def test_mixed_constant_classes(self):

        class FOO(Constants):
            two = VerboseConstant("2", "just two")
            one = SimpleConstant()
            four = ValueConstant(4)
            three = VerboseValueConstant(3, "three", "just three")

        self.assertEqual(
            [x.name for x in FOO.iterconstants()],
            ['two', 'one', 'four', 'three', ]
        )

    def test_mixed_constant_classes_less_generic(self):

        class FOO(Values):
            two = VerboseConstant("2", "just two")
            one = SimpleConstant()
            four = ValueConstant(4)
            three = VerboseValueConstant(3, "three", "just three")

        self.assertEqual(
            [x.name for x in FOO.iterconstants()],
            ['four', 'three', ]
        )

    def test_unbound_constant(self):

        class FOO(Values):
            one = SimpleConstant()
            two = ValueConstant(2)

        self.assertEqual([x.name for x in FOO.iterconstants()], ['two', ])
        self.assertEqual(FOO.one.name, 'one')
        self.assertEqual(FOO.one.full_name, '__UNBOUND__.one')
        self.assertEqual(repr(FOO.one), "<constant '__UNBOUND__.one'>")


class ValuesTestCase(unittest.TestCase):

    def test_get_by_value(self):

        class FOO(Values):
            ONE = ValueConstant(1)
            TWO = ValueConstant(2)
            ONE_DUB = ValueConstant(1)

        self.assertEqual(FOO.get_by_value(1), FOO.ONE)
        self.assertEqual(FOO.get_by_value(2), FOO.get('TWO'))

        with self.assertRaises(ValueError) as cm:
            FOO.get_by_value(3)

        self.assertEqual(
            cm.exception.args[0],
            "Constant with value \"3\" is not present in "
            "\"<constants container 'FOO'>\""
        )

    def test_filter_by_value(self):

        class FOO(Values):
            ONE = ValueConstant(1)
            TWO = ValueConstant(2)
            ONE_DUB2 = ValueConstant(1)
            THREE = ValueConstant(3)
            ONE_DUB1 = ValueConstant(1)

        self.assertEqual(
            [x.name for x in FOO.filter_by_value(1)],
            ['ONE', 'ONE_DUB2', 'ONE_DUB1', ]
        )

    def test_values(self):

        class FOO(Values):
            ONE = ValueConstant(1)
            FOUR = ValueConstant(4)
            THREE = ValueConstant(3)

        self.assertEqual(FOO.values(), [1, 4, 3, ])

    def test_itervalues(self):

        class FOO(Values):
            ONE = ValueConstant(1)
            FOUR = ValueConstant(4)
            THREE = ValueConstant(3)

        self.assertEqual(list(FOO.itervalues()), [1, 4, 3, ])

    def test_to_primitive(self):

        class FOO(Values):
            ONE = ValueConstant(1)
            TWO = ValueConstant(lambda: 2)
            DATE = ValueConstant(date(1999, 12, 31))

        self.assertEqual(
            FOO.ONE.to_primitive(),
            {'name': 'ONE', 'value': 1, }
        )
        self.assertEqual(
            FOO.TWO.to_primitive(),
            {'name': 'TWO', 'value': 2, }
        )
        self.assertEqual(
            FOO.DATE.to_primitive(),
            {'name': 'DATE', 'value': '1999-12-31', }
        )
        self.assertEqual(
            FOO.to_primitive(),
            {
                'name': 'FOO',
                'items': [
                    {'name': 'ONE', 'value': 1, },
                    {'name': 'TWO', 'value': 2, },
                    {'name': 'DATE', 'value': '1999-12-31', },
                ]
            }
        )


class GrouppingTestCase(unittest.TestCase):

    def test_verbose_group(self):

        class FOO(Constants):
            A = SimpleConstant()
            B = VerboseConstant(
                verbose_name="Constant B",
                help_text="Just a group with verbose name"
            ).to_group(
                group_class=Constants,
                C=SimpleConstant(),
                D=SimpleConstant(),
            )

        self.assertEqual(FOO.B.verbose_name, "Constant B")
        self.assertEqual(FOO.B.help_text, "Just a group with verbose name")
        self.assertEqual(FOO.B.names(), ['C', 'D', ])
        self.assertEqual(
            FOO.to_primitive(),
            {
                'name': "FOO",
                'items': [
                    {'name': "A", },
                    {
                        'name': "B",
                        'verbose_name': "Constant B",
                        'help_text': "Just a group with verbose name",
                        'items': [
                            {'name': "C", },
                            {'name': "D", },
                        ],
                    },
                ]
            }
        )

    def test_valuable_group(self):

        class FOO(Constants):
            A = SimpleConstant()
            B = ValueConstant(10).to_group(
                group_class=Constants,
                C=SimpleConstant(),
                D=SimpleConstant(),
            )

        self.assertEqual(FOO.B.value, 10)
        self.assertEqual(FOO.B.names(), ['C', 'D', ])
        self.assertEqual(
            FOO.to_primitive(),
            {
                'name': "FOO",
                'items': [
                    {'name': "A", },
                    {
                        'name': "B",
                        'value': 10,
                        'items': [
                            {'name': "C", },
                            {'name': "D", },
                        ],
                    },
                ]
            }
        )

    def test_valuable_verbose_group(self):

        class FOO(Constants):
            A = SimpleConstant()
            B = VerboseValueConstant(
                value=10,
                verbose_name="Constant B",
                help_text="Just a group with verbose name"
            ).to_group(
                group_class=Constants,
                C=SimpleConstant(),
                D=SimpleConstant(),
            )

        self.assertEqual(FOO.B.value, 10)
        self.assertEqual(FOO.B.verbose_name, "Constant B")
        self.assertEqual(FOO.B.help_text, "Just a group with verbose name")
        self.assertEqual(FOO.B.names(), ['C', 'D', ])
        self.assertEqual(
            FOO.to_primitive(),
            {
                'name': "FOO",
                'items': [
                    {'name': "A", },
                    {
                        'name': "B",
                        'verbose_name': "Constant B",
                        'help_text': "Just a group with verbose name",
                        'value': 10,
                        'items': [
                            {'name': "C", },
                            {'name': "D", },
                        ],
                    },
                ]
            }
        )



# -*- coding: utf-8 -*-

import os
import sys
import unittest

from candv.base import Constant, ConstantsContainer, with_constant_class


class ConstantsContainerTestCase(unittest.TestCase):

    def test_container_instantiation(self):

        with self.assertRaises(TypeError) as cm:
            ConstantsContainer()

        self.assertEqual(
            cm.exception.args[0],
            "\"<constants container 'ConstantsContainer'>\" cannot be "
            "instantiated, because constant containers are not designed for "
            "that."
        )

    def _get_container(self):

        class FOO(ConstantsContainer):
            CONSTANT2 = Constant()
            CONSTANT3 = Constant()
            CONSTANT1 = Constant()

        return FOO

    def test_name(self):
        FOO = self._get_container()
        self.assertEqual(FOO.name, 'FOO')

    def test_full_name(self):
        FOO = self._get_container()
        self.assertEqual(FOO.full_name, 'FOO')

    def test_repr(self):
        FOO = self._get_container()
        self.assertEqual(repr(FOO), "<constants container 'FOO'>")

    def test_names(self):
        FOO = self._get_container()
        self.assertEqual(
            FOO.names(),
            ['CONSTANT2', 'CONSTANT3', 'CONSTANT1', ]
        )

    def test_iternames(self):
        FOO = self._get_container()
        self.assertEqual(
            list(FOO.iternames()),
            ['CONSTANT2', 'CONSTANT3', 'CONSTANT1', ]
        )

    def test_values(self):
        FOO = self._get_container()
        self.assertEqual(
            FOO.values(),
            [FOO.CONSTANT2, FOO.CONSTANT3, FOO.CONSTANT1, ]
        )

    def test_itervalues(self):
        FOO = self._get_container()
        self.assertEqual(
            list(FOO.itervalues()),
            [FOO.CONSTANT2, FOO.CONSTANT3, FOO.CONSTANT1, ]
        )

    def test_constants(self):
        FOO = self._get_container()
        self.assertEqual(
            FOO.constants(),
            [FOO.CONSTANT2, FOO.CONSTANT3, FOO.CONSTANT1, ]
        )

    def test_iterconstants(self):
        FOO = self._get_container()
        self.assertEqual(
            list(FOO.iterconstants()),
            [FOO.CONSTANT2, FOO.CONSTANT3, FOO.CONSTANT1, ]
        )

    def test_items(self):
        FOO = self._get_container()
        self.assertEqual(
            FOO.items(),
            [
                ('CONSTANT2', FOO.CONSTANT2),
                ('CONSTANT3', FOO.CONSTANT3),
                ('CONSTANT1', FOO.CONSTANT1),
            ]
        )

    def test_iteritems(self):
        FOO = self._get_container()
        self.assertEqual(
            list(FOO.iteritems()),
            [
                ('CONSTANT2', FOO.CONSTANT2),
                ('CONSTANT3', FOO.CONSTANT3),
                ('CONSTANT1', FOO.CONSTANT1),
            ]
        )

    def test_iter(self):
        FOO = self._get_container()
        for x, y in zip(FOO, FOO.iternames()):
            self.assertEqual(x, y)

    def test_len(self):
        FOO = self._get_container()
        self.assertEqual(len(FOO), 3)

    def test_contains(self):
        FOO = self._get_container()
        self.assertTrue('CONSTANT2' in FOO)
        self.assertFalse('CONSTANT_X' in FOO)

    def test_has_name(self):
        FOO = self._get_container()
        self.assertTrue(FOO.has_name('CONSTANT2'))
        self.assertFalse(FOO.has_name('CONSTANT_X'))

    def test_get_item(self):
        FOO = self._get_container()
        self.assertEqual(FOO['CONSTANT2'], FOO.CONSTANT2)

        with self.assertRaises(KeyError) as cm:
            FOO['CONSTANT_X']
        self.assertEqual(
            cm.exception.args[0],
            "Constant \"CONSTANT_X\" is not present in "
            "\"<constants container 'FOO'>\""
        )

    def test_get(self):
        FOO = self._get_container()
        self.assertEqual(FOO.get('CONSTANT2'), FOO.CONSTANT2)
        self.assertEqual(FOO.get('CONSTANT_X'), None)

    def test_invalid_constant_class(self):

        with self.assertRaises(TypeError) as cm:

            class FOO(ConstantsContainer):
                constant_class = int

        self.assertEqual(
            cm.exception.args[0],
            "\"{0}\" which is used as \"constant_class\" for "
            "\"<constants container 'FOO'>\" must be derived from "
            "\"<class 'candv.base.Constant'>\".".format(repr(int))
        )

    def test_reuse_of_constant(self):

        with self.assertRaises(ValueError) as cm:

            class A(ConstantsContainer):
                FOO = Constant()
                BAR = Constant()

            class B(ConstantsContainer):
                FOO = Constant()
                BAR = A.BAR

        self.assertEqual(
            cm.exception.args[0],
            "Cannot use \"<constant 'A.BAR'>\" as value for the attribute "
            "\"BAR\" for \"<constants container 'B'>\", because "
            "\"<constant 'A.BAR'>\" already belongs to "
            "\"<constants container 'A'>\"."
        )

    def test_constant_class_mixin_factory(self):

        class SomeConstant(Constant):
            pass

        class FOO(with_constant_class(SomeConstant), ConstantsContainer):
            A = SomeConstant()
            B = SomeConstant()
            C = Constant()

        self.assertEqual(FOO.constant_class, SomeConstant)
        self.assertEqual(FOO.constants(), [FOO.A, FOO.B, ])

    def test_to_primitive(self):

        class FOO(ConstantsContainer):
            ONE = Constant()
            TWO = Constant()
            TREE = Constant()

        self.assertEqual(
            FOO.to_primitive(),
            {
                'name': 'FOO',
                'items': [
                    {'name': 'ONE', },
                    {'name': 'TWO', },
                    {'name': 'TREE', },
                ]
            }
        )


class ConstantTestCase(unittest.TestCase):

    def test_creation_counter(self):
        value = Constant._creation_counter
        Constant()
        self.assertEqual(Constant._creation_counter, value + 1)

    def test_name(self):

        class FOO(ConstantsContainer):
            CONSTANT = Constant()

        self.assertEqual(FOO.CONSTANT.name, 'CONSTANT')

    def test_full_name(self):

        class FOO(ConstantsContainer):
            CONSTANT = Constant()

        self.assertEqual(FOO.CONSTANT.full_name, 'FOO.CONSTANT')

    def test_repr(self):

        class FOO(ConstantsContainer):
            CONSTANT = Constant()

        self.assertEqual(repr(FOO.CONSTANT), "<constant 'FOO.CONSTANT'>")

    def test_container(self):
        constant = Constant()
        self.assertIsNone(constant.container)

        class FOO(ConstantsContainer):
            CONSTANT = Constant()

        self.assertEqual(FOO.CONSTANT.container, FOO)

    def test_to_primitive(self):

        class FOO(ConstantsContainer):
            CONSTANT = Constant()

        self.assertEqual(FOO.CONSTANT.to_primitive(), {'name': 'CONSTANT'})


class GrouppingTestCase(unittest.TestCase):

    def _get_group(self):
        class FOO(ConstantsContainer):
            A = Constant()
            B = Constant().to_group(
                group_class=ConstantsContainer,
                B2=Constant(),
                B0=Constant(),
                B1=Constant(),
            )

        return FOO

    def test_group_name(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.name, 'B')

    def test_group_full_name(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.full_name, 'FOO.B')

    def test_group_repr(self):
        FOO = self._get_group()
        self.assertEqual(repr(FOO.B), "<constants group 'FOO.B'>")

    def test_group_names(self):
        FOO = self._get_group()
        self.assertEqual(
            FOO.B.names(),
            ['B2', 'B0', 'B1', ]
        )

    def test_group_iternames(self):
        FOO = self._get_group()
        self.assertEqual(
            list(FOO.B.iternames()),
            ['B2', 'B0', 'B1', ]
        )

    def test_group_values(self):
        FOO = self._get_group()
        self.assertEqual(
            FOO.B.values(),
            [FOO.B.B2, FOO.B.B0, FOO.B.B1, ]
        )

    def test_group_itervalues(self):
        FOO = self._get_group()
        self.assertEqual(
            list(FOO.B.itervalues()),
            [FOO.B.B2, FOO.B.B0, FOO.B.B1, ]
        )

    def test_group_constants(self):
        FOO = self._get_group()
        self.assertEqual(
            FOO.B.constants(),
            [FOO.B.B2, FOO.B.B0, FOO.B.B1, ]
        )

    def test_group_iterconstants(self):
        FOO = self._get_group()
        self.assertEqual(
            list(FOO.B.iterconstants()),
            [FOO.B.B2, FOO.B.B0, FOO.B.B1, ]
        )

    def test_group_items(self):
        FOO = self._get_group()
        self.assertEqual(
            FOO.B.items(),
            [
                ('B2', FOO.B.B2),
                ('B0', FOO.B.B0),
                ('B1', FOO.B.B1),
            ]
        )

    def test_group_iteritems(self):
        FOO = self._get_group()
        self.assertEqual(
            list(FOO.B.iteritems()),
            [
                ('B2', FOO.B.B2),
                ('B0', FOO.B.B0),
                ('B1', FOO.B.B1),
            ]
        )

    def test_group_iter(self):
        FOO = self._get_group()
        for x, y in zip(FOO.B, FOO.B.iternames()):
            self.assertEqual(x, y)

    def test_group_len(self):
        FOO = self._get_group()
        self.assertEqual(len(FOO.B), 3)

    def test_group_contains(self):
        FOO = self._get_group()
        self.assertTrue('B2' in FOO.B)
        self.assertFalse('B_X' in FOO.B)

    def test_group_has_name(self):
        FOO = self._get_group()
        self.assertTrue(FOO.B.has_name('B2'))
        self.assertFalse(FOO.B.has_name('B_X'))

    def test_group_get_item(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B['B2'], FOO.B.B2)

        with self.assertRaises(KeyError) as cm:
            FOO.B['B_X']

        self.assertEqual(
            cm.exception.args[0],
            "Constant \"B_X\" is not present in "
            "\"<constants group 'FOO.B'>\""
        )

    def test_group_get(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.get('B2'), FOO.B.B2)
        self.assertEqual(FOO.B.get('B_X'), None)

    def test_group_container(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.container, FOO)

    def test_group_as_container(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.B2.container, FOO.B)

    def test_container_names(self):
        FOO = self._get_group()
        self.assertEqual(FOO.names(), ['A', 'B', ])

    def test_get_from_container(self):
        FOO = self._get_group()
        self.assertEqual(FOO['B'], FOO.B)

    def test_group_member_full_name(self):
        FOO = self._get_group()
        self.assertEqual(FOO.B.B2.full_name, 'FOO.B.B2')

    def test_invalid_group(self):
        with self.assertRaises(TypeError) as cm:

            class FOO(ConstantsContainer):
                A = Constant().to_group(
                    group_class=ConstantsContainer,
                    B=Constant(),
                    C=1
                )

        self.assertEqual(
            cm.exception.args[0],
            "\"1\" cannot be a member of a group. Only instances of "
            "\"<class 'candv.base.Constant'>\" or other groups can be."
        )

    def test_to_primitive(self):
        self.assertEqual(
            self._get_group().to_primitive(),
            {
                'name': 'FOO',
                'items': [
                    {'name': 'A', },
                    {
                        'name': 'B',
                        'items': [
                            {'name': 'B2', },
                            {'name': 'B0', },
                            {'name': 'B1', },
                        ]
                    },
                ]
            }
        )

    def test_comparison(self):
        sys.path.insert(
            0,
            os.path.join(
                os.path.dirname(os.path.abspath(__file__)),
                'package',
            )
        )
        from .package.subpackage.constants import CONSTANTS
        from subpackage.constants import CONSTANTS as SUBCONSTANTS

        self.assertNotEqual(CONSTANTS.__module__, SUBCONSTANTS.__module__)
        self.assertEqual(CONSTANTS.PRIMARY, SUBCONSTANTS.PRIMARY)



# -*- coding: utf-8 -*-
"""
This module provides ready-to-use classes for constructing custom constants.
"""

import six

from .base import (
    Constant as SimpleConstant, ConstantsContainer as Constants,
    with_constant_class,
)


class VerboseMixin(object):
    """
    Provides support of verbose names and help texts. Must be placed at the
    left side of non-mixin base classes due to Python's MRO. Arguments must be
    passed as kwargs.

    :argument str verbose_name: optional verbose name
    :argument str help_text: optional description

    **Example**::

        class Foo(object):

            def __init__(self, arg1, arg2, kwarg1=None):
                pass


        class Bar(VerboseMixin, Foo):

            def __init__(self, arg1, arg2, verbose_name=None, help_text=None, kwarg1=None):
                super(Bar, self).__init__(arg1, arg2, verbose_name=verbose_name, help_text=help_text, kwarg1=kwarg1)

    """
    def __init__(self, *args, **kwargs):
        self.verbose_name = kwargs.pop('verbose_name', None)
        self.help_text = kwargs.pop('help_text', None)
        super(VerboseMixin, self).__init__(*args, **kwargs)

    def merge_into_group(self, group):
        """
        Redefines :meth:`~candv.base.Constant.merge_into_group` and adds
        ``verbose_name`` and ``help_text`` attributes to the target group.
        """
        super(VerboseMixin, self).merge_into_group(group)
        group.verbose_name = self.verbose_name
        group.help_text = self.help_text

    def to_primitive(self, context=None):
        """
        .. versionadded:: 1.3.0
        """
        primitive = super(VerboseMixin, self).to_primitive(context)
        to_text = lambda x: six.text_type(x) if x is not None else x
        primitive.update({
            'verbose_name': to_text(self.verbose_name),
            'help_text': to_text(self.help_text),
        })
        return primitive


class VerboseConstant(VerboseMixin, SimpleConstant):
    """
    Constant with optional verbose name and optional description.

    :argument str verbose_name: optional verbose name of the constant
    :argument str help_text: optional description of the constant

    :ivar str verbose_name: verbose name of the constant. Default: ``None``
    :ivar str help_text: verbose description of the constant. Default: ``None``
    """
    def __init__(self, verbose_name=None, help_text=None):
        super(VerboseConstant, self).__init__(verbose_name=verbose_name,
                                              help_text=help_text)


class ValueConstant(SimpleConstant):
    """
    Extended version of :class:`SimpleConstant` which provides support for
    storing values of constants.

    :argument value: a value to attach to constant

    :ivar value: constant's value
    """
    def __init__(self, value):
        super(ValueConstant, self).__init__()
        self.value = value

    def merge_into_group(self, group):
        """
        Redefines :meth:`~candv.base.Constant.merge_into_group` and adds
        ``value`` attribute to the target group.
        """
        super(ValueConstant, self).merge_into_group(group)
        group.value = self.value

    def to_primitive(self, context=None):
        """
        .. versionadded:: 1.3.0
        """
        primitive = super(ValueConstant, self).to_primitive(context)
        value = self.value

        if hasattr(value, 'isoformat'):
            value = value.isoformat()
        elif callable(value):
            value = value()

        primitive['value'] = value
        return primitive


class VerboseValueConstant(VerboseMixin, ValueConstant):
    """
    A constant which can have both verbose name, help text and a value.

    :argument value: a value to attach to the constant
    :argument str verbose_name: optional verbose name of the constant
    :argument str help_text: optional description of the constant

    :ivar value: constant's value
    :ivar str verbose_name: verbose name of the constant. Default: ``None``
    :ivar str help_text: verbose description of the constant. Default: ``None``
    """
    def __init__(self, value, verbose_name=None, help_text=None):
        super(VerboseValueConstant, self).__init__(value,
                                                   verbose_name=verbose_name,
                                                   help_text=help_text)


class Values(with_constant_class(ValueConstant), Constants):
    """
    Constants container which supports getting and filtering constants by their
    values, listing values of all constants in container.
    """

    @classmethod
    def get_by_value(cls, value):
        """
        Get constant by its value.

        :param value: value of the constant to look for
        :returns: first found constant with given value
        :raises ValueError: if no constant in container has given value
        """
        for constant in cls.iterconstants():
            if constant.value == value:
                return constant
        raise ValueError(
            "Constant with value \"{0}\" is not present in \"{1}\""
            .format(value, cls)
        )

    @classmethod
    def filter_by_value(cls, value):
        """
        Get all constants which have given value.

        :param value: value of the constants to look for
        :returns: list of all found constants with given value
        """
        constants = []
        for constant in cls.iterconstants():
            if constant.value == value:
                constants.append(constant)
        return constants

    @classmethod
    def values(cls):
        """
        List values of all constants in the order they were defined.

        :returns: :class:`list` of values

        **Example**::

            >>> from candv import Values, ValueConstant
            >>> class FOO(Values):
            ...     TWO = ValueConstant(2)
            ...     ONE = ValueConstant(1)
            ...     SOME = ValueConstant("some string")
            ...
            >>> FOO.values()
            [2, 1, 'some string']

        .. note::

            Overrides :meth:`~candv.base.ConstantsContainer.values` since
            1.1.2.
        """
        return [x.value for x in cls.iterconstants()]

    @classmethod
    def itervalues(cls):
        """
        Same as :meth:`values` but returns an interator.

        .. note::

            Overrides :meth:`~candv.base.ConstantsContainer.itervalues` since
            1.1.2.
        """
        for constant in cls.iterconstants():
            yield constant.value



# -*- coding: utf-8 -*-
"""
This module defines base constant and base container for constants. All other
stuff must be derived from them.

Each container has :attr:`~ConstantsContainer.constant_class` attribute. It
specifies class of constants which will be defined within contaner.
"""

import six
import types

from collections import OrderedDict


class Constant(object):
    """
    Base class for all constants. Can be merged into a container instance.

    :ivar str name: constant's name. Is set up automatically and is equal to
                    the name of container's attribute
    """

    # Tracks each time a Constant instance is created. Used to retain order.
    _creation_counter = 0

    def __init__(self):
        self.name = None
        self.container = None

        # Increase the creation counter, and save our local copy.
        self._creation_counter = Constant._creation_counter
        Constant._creation_counter += 1

    def _post_init(self, name, container=None):
        """
        Called automatically by container after container's class construction.
        """
        self.name = name
        self.container = container

    def to_group(self, group_class, **group_members):
        """
        Convert a constant into a constants group.

        :param class group_class: a class of group container which will be
                                  created
        :param group_members: unpacked dict which defines group members.

        :returns: a lazy constants group which will be evaluated by container.
                  During group evaluation :meth:`merge_into_group` will be
                  called.

        **Example**::

            from candv import Constants, SimpleConstant

            class FOO(Constants):
                A = SimpleConstant()
                B = SimpleConstant().to_group(
                    group_class=Constants,
                    B2=SimpleConstant(),
                    B0=SimpleConstant(),
                    B1=SimpleConstant(),
            )
        """
        return _LazyConstantsGroup(self, group_class, **group_members)

    def merge_into_group(self, group):
        """
        Called automatically by container after group construction.

        .. note::

            Redefine this method in all derived classes. Attach all custom
            attributes and methods to the group here.

        :param group: an instance of :class:`ConstantsContainer` or it's
                      subclass this constant will be merged into

        :returns: ``None``
        """
        group._creation_counter = self._creation_counter

    @property
    def full_name(self):
        prefix = self.container.full_name if self.container else "__UNBOUND__"
        return "{0}.{1}".format(prefix, self.name)

    def to_primitive(self, context=None):
        """
        .. versionadded:: 1.3.0
        """
        return {'name': self.name, }

    def __repr__(self):
        """
        Return text identifying both which constant this is and which
        collection it belongs to.
        """
        return "<constant '{0}'>".format(self.full_name)

    def __hash__(self):
        """
        .. versionadded:: 1.3.1
        """
        return hash(self.full_name)

    def __eq__(self, other):
        """
        .. versionadded:: 1.3.1
        """
        return (isinstance(other, Constant)
                and (self.full_name == other.full_name))

    def __ne__(self, other):
        """
        .. versionadded:: 1.3.1
        """
        return not (self == other)


class _LazyConstantsGroup(object):

    def __init__(self, constant, group_class, **group_members):
        for name, obj in group_members.items():
            if not isinstance(obj, (Constant, _LazyConstantsGroup)):
                raise TypeError(
                    "\"{0}\" cannot be a member of a group. Only instances of "
                    "\"{1}\" or other groups can be."
                    .format(obj, Constant)
                )
        self.constant = constant
        self.group_class = group_class
        self.group_members = group_members

    def _evaluate(self, parent, name):
        full_name = "{0}.{1}".format(parent.full_name, name)
        self.group_members.update({
            'name': name,
            'full_name': full_name,
            'container': parent,
            '__repr': "<constants group '{0}'>".format(full_name),
        })
        group = type(full_name, (self.group_class, ), self.group_members)
        self.constant.merge_into_group(group)

        to_primitive = self._get_to_primitive(self.constant, group)
        group.to_primitive = types.MethodType(to_primitive, group)

        del self.constant
        del self.group_class
        del self.group_members

        return group

    @staticmethod
    def _get_to_primitive(group, constant):

        group_primitive = group.to_primitive
        constant_primitive = constant.to_primitive

        def to_primitive(self, context=None):
            primitive = group_primitive(context)
            primitive.update(constant_primitive(context))
            return primitive

        return to_primitive


class _ConstantsContainerMeta(type):
    """
    Metaclass for creating constants container classes.
    """
    def __new__(self, class_name, bases, attributes):
        if not 'name' in attributes:
            attributes['name'] = class_name
        if not 'full_name' in attributes:
            attributes['full_name'] = class_name

        __repr = attributes.pop(
            '__repr',
            "<constants container '{0}'>".format(attributes['name'])
        )
        cls = (
            super(_ConstantsContainerMeta, self)
            .__new__(self, class_name, bases, attributes)
        )
        cls.__repr = __repr

        constant_class = getattr(cls, 'constant_class', None)
        if not issubclass(constant_class, Constant):
            raise TypeError(
                "\"{0}\" which is used as \"constant_class\" for \"{1}\" must "
                "be derived from \"{2}\"."
                .format(constant_class, cls, Constant)
            )

        constants = []
        for name, the_object in six.iteritems(attributes):
            if isinstance(the_object, _LazyConstantsGroup):
                group = the_object._evaluate(cls, name)
                setattr(cls, name, group)
                constants.append((name, group))
            elif isinstance(the_object, cls.constant_class):
                if the_object.container is not None:
                    raise ValueError(
                        'Cannot use "{0}" as value for the attribute '
                        '"{1}" for "{2}", because "{0}" already belongs '
                        'to "{3}".'
                        .format(the_object, name, cls, the_object.container)
                    )
                the_object._post_init(name, cls)
                constants.append((name, the_object))
            elif isinstance(the_object, Constant):
                the_object._post_init(name)
        constants.sort(key=lambda x: x[1]._creation_counter)
        cls._constants = OrderedDict(constants)

        return cls

    def __repr__(self):
        return self.__repr

    def __getitem__(self, name):
        """
        Try to get constant by it's name.

        :param str name: name of constant to search for

        :returns: a constant
        :rtype: an instance of :class:`Constant` or it's subclass

        :raises KeyError: if constant name ``name`` is not present in
                          container

        **Example**::

            >>> from candv import Constants, SimpleConstant
            >>> class FOO(Constants):
            ...     foo = SimpleConstant()
            ...     bar = SimpleConstant()
            ...
            >>> FOO['foo']
            <constant 'FOO.foo'>
        """
        try:
            return self._constants[name]
        except KeyError:
            raise KeyError(
                "Constant \"{0}\" is not present in \"{1}\""
                .format(name, self)
            )

    def __contains__(self, name):
        return name in self._constants

    def __len__(self):
        return len(self._constants)

    def __iter__(self):
        return self.iternames()

    def get(self, name, default=None):
        """
        Try to get constant by it's name or fallback to default.

        :param str name: name of constant to search for
        :param default: an object returned by default if constant with a given
                        name is not present in the container

        :returns: a constant or a default value
        :rtype: an instance of :class:`Constant` or it's subclass, or `default`
                value

        **Example**::

            >>> from candv import Constants, SimpleConstant
            >>> class FOO(Constants):
            ...     foo = SimpleConstant()
            ...     bar = SimpleConstant()
            ...
            >>> FOO.get('foo')
            <constant 'FOO.foo'>
            >>> FOO.get('xxx')
            >>>
            >>> FOO.get('xxx', default=123)
            123
        """
        return self[name] if name in self else default

    def has_name(self, name):
        """
        Check if container has a constant with a given name.

        :param str name: a constant's name to check

        :returns: ``True`` if given name belongs to container,
                  ``False`` otherwise
        :rtype: :class:`bool`
        """
        return name in self

    def names(self):
        """
        List all names of constants within container.

        :returns: a list of constant names in order constants were defined
        :rtype: :class:`list` of strings

        **Example**::

            >>> from candv import Constants, SimpleConstant
            >>> class FOO(Constants):
            ...     foo = SimpleConstant()
            ...     bar = SimpleConstant()
            ...
            >>> FOO.names()
            ['foo', 'bar']
        """
        return list(self.iternames())

    def iternames(self):
        """
        Same as :meth:`names` but returns an interator.
        """
        return six.iterkeys(self._constants)

    def constants(self):
        """
        List all constants in container.

        :returns: list of constants in order they were defined
        :rtype: :class:`list`

        **Example**::

            >>> from candv import Constants, SimpleConstant
            >>> class FOO(Constants):
            ...     foo = SimpleConstant()
            ...     bar = SimpleConstant()
            ...
            >>> [x.name for x in FOO.constants()]
            ['foo', 'bar']
        """
        return list(self.iterconstants())

    def iterconstants(self):
        """
        Same as :meth:`constants` but returns an interator.
        """
        return six.itervalues(self._constants)

    def items(self):
        """
        Get list of constants with their names.

        :returns: list of constants with their names in order they were
                  defined. Each element in list is a :class:`tuple` in format
                  ``(name, constant)``.
        :rtype: :class:`list`

        **Example**::

            >>> from candv import Constants, SimpleConstant
            >>> class FOO(Constants):
            ...     foo = SimpleConstant()
            ...     bar = SimpleConstant()
            ...
            >>> FOO.items()
            [('foo', <constant 'FOO.foo'>), ('bar', <constant 'FOO.bar'>)]
        """
        return list(self.iteritems())

    def iteritems(self):
        """
        Same as :meth:`items` but returns an interator.
        """
        return six.iteritems(self._constants)

    #: .. versionadded:: 1.1.2
    #:
    #: Alias for :meth:`constants`.
    #: Added for consistency with dictionaries. Use :class:`~candv.Values` and
    #: :meth:`~candv.Values.values` if you need to have constants with real
    #: values.
    values = constants

    #: .. versionadded:: 1.1.2
    #:
    #: Alias for :meth:`iterconstants`.
    #: Added for consistency with dictionaries. Use :class:`~candv.Values` and
    #: :meth:`~candv.Values.itervalues` if you need to have constants with real
    #: values.
    itervalues = iterconstants

    def to_primitive(self, context=None):
        """
        .. versionadded:: 1.3.0
        """
        return {
            'name': self.name,
            'items': [x.to_primitive(context) for x in self.iterconstants()]
        }


@six.add_metaclass(_ConstantsContainerMeta)
class ConstantsContainer(object):
    """
    Base class for creating constants containers. Each constant defined within
    container will remember it's creation order. See an example in
    :meth:`constants`.

    :cvar constant_class: stores a class of constants which can be stored by
                          container. This attribute **MUST** be set up manually
                          when you define a new container type. Otherwise
                          container will not be initialized. Default: ``None``
    :raises TypeError: if you try to create an instance of container.
                       Containers are singletons and they cannot be
                       instantiated. Their attributes must be used directly.
    """

    #: Defines a top-level class of constants which can be stored by container
    constant_class = Constant

    def __new__(cls):
        """
        Classes representing constants containers are not intended to be
        instantiated.

        The class object itself is used directly.
        """
        raise TypeError(
            "\"{0}\" cannot be instantiated, because constant containers are "
            "not designed for that."
            .format(cls)
        )


def with_constant_class(the_class):
    """
    A mixin factory which allows to set constant class for constants container
    outside container itself. This may help to create more readable container
    definition, e.g.:

        >>> from candv import Constants, SimpleConstant, with_constant_class
        >>>
        >>> class SomeConstant(SimpleConstant):
        ...     pass
        ...
        >>> class FOO(with_constant_class(SomeConstant), Constants):
        ...     A = SomeConstant()
        ...     B = SomeConstant()
        ...
        >>> FOO.constant_class
        <class '__main__.SomeConstant'>
    """
    class ConstantContainerMixin(object):
        constant_class = the_class

    return ConstantContainerMixin



# -*- coding: utf-8 -*-

import os

from setuptools import setup, find_packages


__here__ = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(__here__, 'README.rst')).read()
REQUIREMENTS = [
    i.strip() for i in
    open(os.path.join(__here__, 'requirements', 'dist.txt')).readlines()
]

setup(
    name='candv',
    version='1.3.1',
    description="Constants and Values: create grouped non-standard named "
                "constants, add values, verbose names, help texts or anything "
                "you like to them",
    long_description=README,
    keywords=[
        'constants', 'values', 'structures', 'choices',
    ],
    license='LGPLv3',
    url='https://github.com/oblalex/candv',
    author='Alexander Oblovatniy',
    author_email='oblovatniy@gmail.com',
    packages=find_packages(),
    install_requires=REQUIREMENTS,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries',
    ],
    platforms=[
        'any',
    ],
)
